/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;

/**
 * A software implementation of a LFSR-based (linear frequency shift register) pseudo-random number generator.
 * @author Naomi (boyfailure.dev)
 * @since 1.0
 */
public class LFSR {

    private static boolean[] originalArray = new boolean[] {true,false,true,true,true,true,true,true,false,true,false,false,false,true,true,false,false};

    private boolean[] bitArray;
    private int bitCount;
    private int bitsReturned;
    private int firstXorIndex;
    private int secondXorIndex;

    private Timer lfsrTimer;

    public LFSR(int bitCount, int bitsReturned, int firstXorIndex, int secondXorIndex) {
        this.bitCount = bitCount;
        this.bitsReturned = bitsReturned;
        this.firstXorIndex = firstXorIndex;
        this.secondXorIndex = secondXorIndex;
        if (bitsReturned > bitCount) {bitsReturned = bitCount;}
        bitArray = new boolean[bitCount];
        for (int i = 0; i < bitCount; i++) {
            if (i < 17) {bitArray[i] = LFSR.originalArray[i];}
            else {
                bitArray[i] = bitArray[i - 3] ^ bitArray[i - 8];
            }
        }
        this.lfsrTimer = new Timer(1, new ActionListener(){public void actionPerformed(ActionEvent e) {LFSR.this.shift();}});
        this.lfsrTimer.start();
    }
    public LFSR() {
        this(17, 8, 11, 16);
    }

    public int getDecimalValue() {
        int regValue = 0;
        int bitIndex = this.bitCount - 1;

        for (int i = 0; i < this.bitsReturned; i++) {
            if (this.bitArray[bitIndex]) {
                regValue += Math.pow(2, i);
            }
            bitIndex--;
        }

        return regValue;
    }
    public int getShiftedDecimalValue() {
        this.shift();
        return this.getDecimalValue();
    }

    public void shift() {
        for (int i = (this.bitCount - 1); i > 0; i--) {
            this.bitArray[i] = this.bitArray[i - 1];
        }
        this.bitArray[0] = this.bitArray[this.firstXorIndex] ^ this.bitArray[this.secondXorIndex];
    }

    public String getBits() {
        String result = "";
        for (int i = 0; i < this.bitCount; i++) {
            if (this.bitArray[i] == true) {
                result += "1";
            }
            else {
                result += "0";
            }
        }
        return result;
    }

}